"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PDFEngines = void 0;
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const path_1 = tslib_1.__importDefault(require("path"));
const form_data_1 = tslib_1.__importDefault(require("form-data"));
const main_config_1 = require("../main.config");
const common_1 = require("../common");
const pdf_engines_utils_1 = require("./utils/pdf-engines.utils");
/**
 * Class uses PDF engines for various operations such as merging and conversion.
 */
class PDFEngines {
    /**
     * Merges multiple PDF files into a single PDF document.
     *
     * @param {Object} options - Options for the merge operation.
     * @param {PathLikeOrReadStream[]} options.files - An array of PathLikes or ReadStreams to the PDF files to be merged.
     * @returns {Promise<Buffer>} A Promise resolving to the merged PDF content as a Buffer.
     */
    static merge(_a) {
        return tslib_1.__awaiter(this, arguments, void 0, function* ({ files, pdfa, pdfUA, metadata }) {
            const data = new form_data_1.default();
            yield pdf_engines_utils_1.PDFEnginesUtils.addFiles(files, data);
            yield pdf_engines_utils_1.PDFEnginesUtils.customize(data, {
                pdfa,
                pdfUA,
                metadata
            });
            const endpoint = `${main_config_1.Chromiumly.GOTENBERG_ENDPOINT}/${main_config_1.Chromiumly.PDF_ENGINES_PATH}/${main_config_1.Chromiumly.PDF_ENGINE_ROUTES.merge}`;
            return common_1.GotenbergUtils.fetch(endpoint, data, main_config_1.Chromiumly.GOTENBERG_API_BASIC_AUTH_USERNAME, main_config_1.Chromiumly.GOTENBERG_API_BASIC_AUTH_PASSWORD);
        });
    }
    /**
     * Converts various document formats to PDF.
     *
     * @param {Object} options - Options for the conversion operation.
     * @param {PathLikeOrReadStream[]} options.files - An array of PathLikes or ReadStreams to the files to be converted to PDF.
     * @param {pdfa} [options.pdfa] - PDF format options.
     * @param {boolean} [options.pdfUA] - Indicates whether to generate PDF/UA compliant output.
     * @returns {Promise<Buffer>} A Promise resolving to the converted PDF content as a Buffer.
     */
    static convert(_a) {
        return tslib_1.__awaiter(this, arguments, void 0, function* ({ files, pdfa, pdfUA }) {
            const data = new form_data_1.default();
            yield pdf_engines_utils_1.PDFEnginesUtils.addFiles(files, data);
            yield pdf_engines_utils_1.PDFEnginesUtils.customize(data, {
                pdfa,
                pdfUA
            });
            const endpoint = `${main_config_1.Chromiumly.GOTENBERG_ENDPOINT}/${main_config_1.Chromiumly.LIBRE_OFFICE_PATH}/${main_config_1.Chromiumly.LIBRE_OFFICE_ROUTES.convert}`;
            return common_1.GotenbergUtils.fetch(endpoint, data, main_config_1.Chromiumly.GOTENBERG_API_BASIC_AUTH_USERNAME, main_config_1.Chromiumly.GOTENBERG_API_BASIC_AUTH_PASSWORD);
        });
    }
    /**
     * Reads metadata from the provided files.
     *
     * @param {PathLikeOrReadStream[]} files An array of PathLikes or ReadStreams to the PDF files.
     * @returns {Promise<Buffer>} A Promise resolving to the metadata buffer.
     */
    static readMetadata(files) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = new form_data_1.default();
            yield pdf_engines_utils_1.PDFEnginesUtils.addFiles(files, data);
            const endpoint = `${main_config_1.Chromiumly.GOTENBERG_ENDPOINT}/${main_config_1.Chromiumly.PDF_ENGINES_PATH}/${main_config_1.Chromiumly.PDF_ENGINE_ROUTES.readMetadata}`;
            return common_1.GotenbergUtils.fetch(endpoint, data, main_config_1.Chromiumly.GOTENBERG_API_BASIC_AUTH_USERNAME, main_config_1.Chromiumly.GOTENBERG_API_BASIC_AUTH_PASSWORD);
        });
    }
    /**
     * Writes metadata to the provided PDF files.
     *
     * @param {PathLikeOrReadStream[]} files - An array of PathLikes or ReadStreams to the PDF files.
     * @param {Metadata} metadata - Metadata to be written.
     * @returns {Promise<Buffer>} A Promise that resolves to the PDF file containing metadata as a buffer.
     */
    static writeMetadata(_a) {
        return tslib_1.__awaiter(this, arguments, void 0, function* ({ files, metadata }) {
            const data = new form_data_1.default();
            data.append('metadata', JSON.stringify(metadata));
            yield pdf_engines_utils_1.PDFEnginesUtils.addFiles(files, data);
            const endpoint = `${main_config_1.Chromiumly.GOTENBERG_ENDPOINT}/${main_config_1.Chromiumly.PDF_ENGINES_PATH}/${main_config_1.Chromiumly.PDF_ENGINE_ROUTES.writeMetadata}`;
            return common_1.GotenbergUtils.fetch(endpoint, data, main_config_1.Chromiumly.GOTENBERG_API_BASIC_AUTH_USERNAME, main_config_1.Chromiumly.GOTENBERG_API_BASIC_AUTH_PASSWORD);
        });
    }
    /**
     * Generates a PDF file from a buffer and saves it to the "__generated__" directory.
     *
     * @param {string} filename - The filename for the generated PDF.
     * @param {Buffer} buffer - The PDF content as a Buffer.
     * @returns {Promise<void>} A Promise that resolves once the file is generated and saved.
     */
    static generate(filename, buffer) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const __generated__ = path_1.default.resolve(process.cwd(), '__generated__');
            yield fs_1.promises.mkdir(path_1.default.resolve(__generated__), { recursive: true });
            yield fs_1.promises.writeFile(path_1.default.resolve(__generated__, filename), buffer);
        });
    }
}
exports.PDFEngines = PDFEngines;
//# sourceMappingURL=pdf-engines.js.map